%example: [matches,m,descriptor1,descriptor2]=Feature_matching(img1,img2,kp1,kp2,4,4,30,1,'EP','4*4',100,0.75,0,1);
function [matches,m,descriptor1,descriptor2,kp1,kp2]=Feature_matching(img1,img2,kp1,kp2,n1,n2,r,dr,method,m,TH1,TH2,rotate,draw,Q)
%Feature_matching Match keypoints in two images
%Input
%   img1:       an input image as query image
%   img2:       an input image as refrence image
%   kp1:        keypoints ([yi xi Ri]) in img1
%   kp2:        keypoints ([yi xi Ri]) in img2
%   n1:         the lowest degree of EP used
%   n2:         the highest degree of EP used
%   r:          the fixed size that local images resized to
%   dr:         the width of rings while using ring-subregions (m=='circle')
%   method:     "EP" or "SURF"
%   m:          types of sub-region ("4*4" is usually the best)
%   Th1,TH2:    p1 and p2 are matched, if dis(p1,p2)_1st<=Th1 && dis(p1,p2)_1st/dis(p1,p2)_2nd<=TH2
%   rotate:     orientation is to be assigned if rotate==1
%   draw:       the matching result is displayed if draw==1
%   Q:          Q will be computed if not given in input
%Output
%   matches:    [Y1    X1    Y2    X2    dis(p1,p2)_1st  (dis(p1,p2)_1st/dis(p1,p2)_2nd)  (value of 0 means it is matched)] 
%   m:          [(index of point with minimum distance)   (dis(p1,p2)_1st/dis(p1,p2)_2nd)], useless
%   descriptor1:feature vectors of kp1
%   descriptor2:feature vectors of kp2
%   kp1,kp2:    keypoints output


% img1bw=double(rgb2gray(img1))/255;
% img2bw=double(rgb2gray(img2))/255;
if(nargin<15)
    switch method
        case 'SURF'
            Q=SURF_basis(r);
        case 'EP'
            Q=EP_basis(r,n1,n2);
%         case 'DoG'
%             Q=DoG_basis(r,N);
%         case 'Gabor'
%             Q=Gabor_basis(r,N);
    end
    Q=basisTrans(Q,r,dr,method,m);
end
[descriptor1,kp1]=descriptor(img1,kp1,method,n1,n2,r,dr,m,rotate,Q);
% matches=0;
% m=0;
% descriptor2=0;
[descriptor2,kp2]=descriptor(img2,kp2,method,n1,n2,r,dr,m,rotate,Q);
% outputDescriptor(kp1,descriptor1,'descriptor_data\descriptor1.txt');
% outputDescriptor(kp2,descriptor2,'descriptor_data\descriptor2.txt');
[matches,m]=do_matching(img1,img2,kp1,kp2,descriptor1,descriptor2,TH1,TH2,draw);

