%Example: [kp1]=myDetector(img1,0.02,3,30,1);
function [kp,I]=myDetector(img,TH,r,R,display)
%myDetector Detect corners based on Harris Measure.
%Input
%   TH:     if the value of Harris Measure (HM) of one pixel is less than TH*HM_max, the pixel cannot be detected even it is a local maximum.
%   r:      the region-size to detect a local maximum of HM. Only one corner can be detected in it.
%   R:      a fixed radius to be assigned to all corners as scale.
%Output
%   kp1:    [Y X R]

if(nargin<5)
    display=0;
end
% tic
[lines,rows]=size(img);
[kp_temp,I]=kp_harris(img,TH,r);
kp_temp=kp_temp(kp_temp(:,1)>R&(lines-kp_temp(:,1))>=R&kp_temp(:,2)>R&(rows-kp_temp(:,2))>=R,:);
n_kp=size(kp_temp,1);
kp=[kp_temp,ones(n_kp,1)*R];
% toc
I=img;
for r=1: n_kp
    I(kp(r,1)-2:kp(r,1)+2,kp(r,2)-2)=1;
    I(kp(r,1)-2:kp(r,1)+2,kp(r,2)+2)=1;
    I(kp(r,1)-2,kp(r,2)-2:kp(r,2)+2)=1;
    I(kp(r,1)+2,kp(r,2)-2:kp(r,2)+2)=1;
end
if(display==1)
    figure;
    imagesc(I);
    colormap gray;
end